@@ -8,7 +8,20 @@ import math
 import ctypes
 import logging
 from PySpeedMacro.PSMMouse import WinMouse
+import PySpeedMacro.PSMInterface
 import colorsys
+import os
+import shutil
+import subprocess
+import sys
+import tempfile
+from PIL import Image
+import mss
+import concurrent
+
+platform = sys.platform
+is_darwin = platform == "darwin"
+is_win32 = platform == "win32"
 
 wm = WinMouse()
 
@@ -62,10 +75,53 @@ def timer(func):
 
     return wrapper
 
+def capture_screenshot(region=None, grayscale=False):
+    if region is None:
+        # Capture the whole screen
+        with mss.mss() as sct:
+            monitor = sct.monitors[1]
+            screenshot = sct.grab(monitor)
+    else:
+        # Capture a specific region
+        x, y, width, height = region
+        with mss.mss() as sct:
+            screenshot = sct.grab({'left': x, 'top': y, 'width': width, 'height': height})
 
-def Screenshot(region=None):
+    # Convert the screenshot to an OpenCV image
+    img = np.array(screenshot)
+    img = cv2.cvtColor(img, cv2.COLOR_BGRA2BGR)
 
-    """
+    # Convert to grayscale if required
+    if grayscale:
+        img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
+
+    return img
+
+def Screenshot(region=None, grayscale=False, num_threads=16):
+    executor = concurrent.futures.ThreadPoolExecutor(max_workers=num_threads)
+    with mss.mss() as sct:
+        if region is None:
+            monitor = sct.monitors[1]
+            monitor_info = monitor
+            regions = [(monitor_info['left'], monitor_info['top'], monitor_info['width'], monitor_info['height'])]
+        else:
+            regions = [region]
+
+        # Submit screenshot capture tasks to the thread pool
+        futures = []
+        for reg in regions:
+            future = executor.submit(capture_screenshot, reg, grayscale)
+            futures.append(future)
+
+        # Wait for all the tasks to complete and retrieve the results
+        screenshots = [future.result() for future in futures]
+
+        # Merge multiple screenshots into a single image
+        merged_screenshot = np.concatenate(screenshots, axis=1) if len(screenshots) > 1 else screenshots[0]
+
+        return merged_screenshot
+
+"""
 
     Description: Quick and easy Screenshot method. 
 
@@ -83,21 +139,6 @@ def Screenshot(region=None):
 
     """
 
-    if region is not None:
-        left, top, size_x, size_y = region
-        right = left + size_x
-        bottom = top + size_y
-        screen = ImageGrab.grab(bbox=(left, top, right, bottom))
-        #print(f"start at {left}, {top}. ends at: {right}, {bottom}")
-
-    else:
-
-
-        screen = ImageGrab.grab()
-
-    return np.array(screen)
-
-
 def lerp(a, b, t):
 
     """
@@ -336,7 +377,7 @@ def locateOnScreen(path, confidence=1, directory='.', region=None, grayscale=Fal
 
 
         fullPath = os.path.join(directory, path)
-        screen = Screenshot(region)
+        screen = Screenshot(region, grayscale=grayscale)
         image = cv2.imread(fullPath)
 
         if grayscale:
@@ -345,7 +386,6 @@ def locateOnScreen(path, confidence=1, directory='.', region=None, grayscale=Fal
             If grayscale is True, then convert image to grayscale.
             """
 
-            screen = cv2.cvtColor(screen, cv2.COLOR_RGB2GRAY)
             image = cv2.cvtColor(image, cv2.COLOR_RGB2GRAY)
 
         else:
@@ -379,7 +419,7 @@ def locateOnScreen(path, confidence=1, directory='.', region=None, grayscale=Fal
         return None
 
 
-def locateCenter(path, confidence=1, directory='.', region=None, grayscale=False):
+def locateCenter(path, confidence=1, directory='.', region=None, grayscale=False, method = cv2.TM_CCOEFF_NORMED):
 
     """
 
@@ -405,19 +445,21 @@ def locateCenter(path, confidence=1, directory='.', region=None, grayscale=False
 
 
         fullPath = os.path.join(directory, path)
-        screen = Screenshot(region)
+        screen = Screenshot(region, grayscale=grayscale)
         image = cv2.imread(fullPath)
 
         if grayscale:
 
-            screen = cv2.cvtColor(screen, cv2.COLOR_RGB2GRAY)
+            """
+            If grayscale is True, then convert image to grayscale.
+            """
             image = cv2.cvtColor(image, cv2.COLOR_RGB2GRAY)
 
         else:
 
             image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
 
-        result = cv2.matchTemplate(screen, image, cv2.TM_CCOEFF_NORMED)
+        result = cv2.matchTemplate(screen, image, method)
 
         min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(result)
 
@@ -437,9 +479,7 @@ def locateCenter(path, confidence=1, directory='.', region=None, grayscale=False
 
             return None
 
-
-    except cv2.error as e:
-
+    except Exception as e:
         print(e)
         return None
 
@@ -476,12 +516,11 @@ def locateAndMove(path, confidence=1, directory='.', region=None, grayscale=Fals
 
             raise ValueError("Height of the screenshot region must be a positive number.")
 
-        screen = Screenshot(region)
+        screen = Screenshot(region, grayscale=grayscale)
         image = cv2.imread(fullPath)
 
         if grayscale:
 
-            screen = cv2.cvtColor(screen, cv2.COLOR_RGB2GRAY)
             image = cv2.cvtColor(image, cv2.COLOR_RGB2GRAY)
 
         else:
@@ -521,15 +560,88 @@ def locateAndMove(path, confidence=1, directory='.', region=None, grayscale=Fals
         print(e)
         return None
 
+def getPixelRGB(x, y):
 
-def getPixel(x, y):
+    screen = np.array(ImageGrab.grab())
 
-    im = ImageGrab.grab()
-
-    r, g, b = im.getpixel((x, y))
+    r, g, b = screen[y, x, :]
 
     return (r, g, b)
 
+def getPixelHSV(x, y):
+
+    screen = np.array(ImageGrab.grab())
+    hsv_image = cv2.cvtColor(np.array(screen), cv2.COLOR_RGB2HSV)
+
+    hsv = hsv_image[y, x]
+    return (hsv[0], hsv[1], hsv[2])
+
+
+def grab(bbox=None, include_layered_windows=False, all_screens=False, xdisplay=None):
+    if xdisplay is None:
+        if is_darwin:
+            fh, filepath = tempfile.mkstemp(".png")
+            os.close(fh)
+            args = ["screencapture"]
+            if bbox:
+                left, top, right, bottom = bbox
+                args += ["-R", f"{left},{top},{right - left},{bottom - top}"]
+            subprocess.call(args + ["-x", filepath])
+            im = Image.open(filepath)
+            im.load()
+            os.unlink(filepath)
+            if bbox:
+                im_resized = im.resize((right - left, bottom - top))
+                im.close()
+                return im_resized
+            return im
+        elif is_win32:
+            offset, size, data = Image.core.grabscreen_win32(include_layered_windows, all_screens)
+            im = Image.frombytes(
+                "RGB",
+                size,
+                data,
+                "raw",
+                "BGR",
+                (size[0] * 3 + 3) & -4,
+                -1,
+            )
+            if bbox:
+                x0, y0 = offset
+                left, top, right, bottom = bbox
+                im = im.crop((left - x0, top - y0, right - x0, bottom - y0))
+            return im
+    try:
+        if not Image.core.HAVE_XCB:
+            raise OSError("Pillow was built without XCB support")
+        size, data = Image.core.grabscreen_x11(xdisplay)
+    except OSError:
+        if xdisplay is None and platform not in ("darwin", "win32") and shutil.which("gnome-screenshot"):
+            fh, filepath = tempfile.mkstemp(".png")
+            os.close(fh)
+            subprocess.call(["gnome-screenshot", "-f", filepath])
+            im = Image.open(filepath)
+            im.load()
+            os.unlink(filepath)
+            if bbox:
+                im_cropped = im.crop(bbox)
+                im.close()
+                return im_cropped
+            return im
+        else:
+            raise
+    else:
+        im = Image.frombytes("RGB", size, data, "raw", "BGRX", size[0] * 4, 1)
+        if bbox:
+            im = im.crop(bbox)
+        return im
+
+
+
+
+
+
+
 
 @timer
 def getHue(region):
@@ -556,6 +668,9 @@ def getHue(region):
 
 
 def setWindow(name):
+    """
+    Name = Window name : Type = String
+    """
     # Get the window handle for the specified window name
     handle = win32gui.FindWindow(None, name)
 
@@ -564,8 +679,12 @@ def setWindow(name):
         print(f"No window found with name '{name}'")
         return
 
-    # Set the current active window to the specified window
-    ctypes.windll.user32.SetForegroundWindow(handle)
-    win32gui.ShowWindow(handle, win32con.SW_RESTORE)
+    # Check if the window is currently hidden
+    if win32gui.IsIconic(handle):
+        # Window is hidden, so restore it
+        win32gui.ShowWindow(handle, win32con.SW_RESTORE)
+    else:
+        # Window is already active, so just focus on it
+        ctypes.windll.user32.SetForegroundWindow(handle)
 
-    print(f"Window '{name}' activated")
\ No newline at end of file
+    print(f"Window '{name}' activated")
